@@ -10,11 +10,11 @@ Net::SNMP::Mixin - mixin framework for Net::SNMP
 
 =head1 VERSION
 
-Version 0.01_02
+Version 0.07_01
 
 =cut
 
-our $VERSION = '0.01_02';
+our $VERSION = '0.07_01';
 
 =head1 ABSTRACT
 
@@ -40,10 +40,16 @@ use Package::Reaper;
 #
 # this module export config
 #
+my @mixin_methods;
+
+BEGIN {
+  @mixin_methods = ( qw/ mixer init_mixins /);
+}
+
 use Sub::Exporter -setup => {
   into    => 'Net::SNMP',
-  exports => [qw/mixer init_mixins/],
-  groups  => { default => [qw/mixer init_mixins/] }
+  exports => [@mixin_methods],
+  groups  => { default => [@mixin_methods] }
 };
 
 # needed for housekeeping of already mixed in modules
@@ -136,6 +142,9 @@ sub mixer {
   return $self;
 }
 
+#
+# Mix the module into Net::SNMP with the help of Sub::Exporter.
+#
 sub _class_mixer {
   my ( $class, $mixin ) = @_;
 
@@ -143,6 +152,11 @@ sub _class_mixer {
   Carp::croak $@ if $@;
 }
 
+#
+# Create a new package as a subclass of Net::SNMP
+# Rebless $session in the new package.
+# Mix the module into the new package with the help of Sub::Exporter.
+#
 sub _obj_mixer {
   my ( $session, $mixin )      = @_;
   my ( $package, $pkg_reaper ) = _make_package($session);
@@ -159,10 +173,9 @@ sub _obj_mixer {
 }
 
 #
-# make unique mixin subclass for this session with name
-# Net::SNMP::<refaddr $session> und make it a subclass
-# of Net::SNMP. Mix all mixin modules into this only package
-# for this session.
+# Make unique mixin subclass for this session with name.
+# Net::SNMP::<refaddr $session> und make it a subclass of Net::SNMP.
+# Arm a package reaper, see perldoc Package::Reaper.
 #
 sub _make_package {
   my $session  = shift;
@@ -191,11 +204,15 @@ sub _make_package {
   $session->init_mixins();
   $session->init_mixins(1);
 
-This method redispatches to every I<< _init() >> methods in the loaded mixin modules. The raw SNMP values for the mixins are loaded during this call - or via callbacks during the snmp_dispatcher event loop for nonblocking sessions - and stored in the object space. The mixed methods deliver afterwards cooked meal from these values.
+This method redispatches to every I<< _init() >> method in the loaded mixin modules. The raw SNMP values for the mixins are loaded during this call - or via callbacks during the snmp_dispatcher event loop for nonblocking sessions - and stored in the object space. The mixed methods deliver afterwards cooked meal from these values.
 
 The MIB values are reloaded for the mixins if the argument $reload is true. It's an error calling this method twice without forcing $reload.
 
-Returns nothing, dies on error.
+This method should be called in void context. In order to check successfull initialization the Net::SNMP error method I<< $session->error() >> should be checked. Please use the following idiom:
+
+  $session->init_mixins;
+  snmp_dispatcher if $Net::SNMP::NONBLOCKING;
+  die $session->error if $session->error;
 
 =cut
 
@@ -205,17 +222,49 @@ sub init_mixins {
   Carp::croak "pure instance method called as class method,"
     unless ref $session;
 
-  my @class_mixins    = keys %register_class_mixins;
-  my @instance_mixins = keys %{ $session->{$prefix}{mixins} };
+  my @class_mixins    = _get_class_mixins();
+  my @instance_mixins = _get_instance_mixins($session);
+  my @all_mixins      = (@class_mixins, @instance_mixins);
 
-  foreach my $mixin ( @class_mixins, @instance_mixins ) {
-    eval "${mixin}::_init(\$session, \$reload)";
+  unless ( scalar @all_mixins ) {
+    Carp::carp "please use first the mixer() method, nothing to init\n";
+    return;
+  }
+
+  # for each mixin module ...
+  foreach my $mixin (@all_mixins) {
+
+    # call the _init() method in module $mixin
+    eval "\$session->${mixin}::_init(\$reload)";
     Carp::croak $@ if $@;
   }
 }
 
+#
+# _get_class_mixins()
+# Returns a list of already mixed-in modules into the Net::SNMP class.
+#
+sub _get_class_mixins {
+    return keys %register_class_mixins;
+}
+
+#
+# _get_instance_mixins($session) >>
+# Returns a list of already mixed-in modules into the namespace of $session.
+#
+sub _get_instance_mixins {
+  my $session = shift;
+
+  return keys %{ $session->{$prefix}{mixins} }
+    if defined $session->{$prefix}{mixins};
+
+  return;
+}
+
 =head1 GUIDELINES FOR MIXIN AUTHORS
 
+See the L<< Net::SNMP::Mixin::System >> module as a blueprint for a simple mixin module.
+
 As a mixin-module author you must respect the following design guidelines:
 
 =over 4
@@ -226,7 +275,7 @@ Write more separate mixin-modules instead of 'one module fits all'.
 
 =item *
 
-Don't build interdependancies to other mixin-modules.
+Don't build mutual dependencies with other mixin-modules.
 
 =item *
 
@@ -234,15 +283,21 @@ In no circumstance change the given attributes of the calling Net::SNMP session
 
 =item *
 
-Use Sub::Exporter and export the mixin methods by default.
+Don't assume the translation of the SNMP values by default. Due to the asynchronous nature of the SNMP calls, you can't rely on the output of $session->translate. If you need a special representation of a value, you have to check the values itself and perhaps translate or untranslate it when needed. See the source of Net::SNMP::Mixin::Dot1qVlanStatic for an example.
 
 =item *
 
 Implement the I<< _init() >> method and fetch SNMP values only during this call. If the session instance is nonblocking use a callback to work properly with the I<< snmp_dispatcher() >> event loop. In no circumstance load additonal SNMP values outside the  I<< _init() >> method.
 
-=back
+=item *
 
-See the L<< Net::SNMP::Mixin::System >> module as a blueprint for a simple mixin module.
+Don't die() on SNMP errors during I<< _init() >>, just return premature with no value. The caller is responsible to check the I<< $session->error() >> method.
+
+=item *
+
+Use Sub::Exporter and export the mixin methods by default.
+
+=back
 
 =head1 DEVELOPER INFORMATION
 
@@ -258,41 +313,17 @@ L<< Sub::Exporter >>, and the Net::SNMP::Mixin::... documentations for more deta
 
 L<Net::SNMP>, L<Sub::Exporter>, L<Package::Generator>, L<Package::Reaper>
 
-=head1 BUGS
+=head1 BUGS, PATCHES & FIXES
 
-Please report any bugs or feature requests to
-C<bug-net-snmp-mixin at rt.cpan.org>, or through the web interface at
-L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=Net-SNMP-Mixin>.
-I will be notified, and then you'll automatically be notified of progress on
-your bug as I make changes.
+There are no known bugs at the time of this release. However, if you spot a bug or are experiencing difficulties that are not explained within the POD documentation, please submit a bug to the RT system (see link below). However, it would help greatly if you are able to pinpoint problems or even supply a patch. 
 
-=head1 SUPPORT
+Fixes are dependant upon their severity and my availablity. Should a fix not be forthcoming, please feel free to (politely) remind me by sending an email to gaissmai@cpan.org .
 
-You can find documentation for this module with the perldoc command.
+  RT: http://rt.cpan.org/Public/Dist/Display.html?Name=Net-SNMP-Mixin
 
-    perldoc Net::SNMP::Mixin
+=head1 AUTHOR
 
-You can also look for information at:
-
-=over 4
-
-=item * AnnoCPAN: Annotated CPAN documentation
-
-L<http://annocpan.org/dist/Net-SNMP-Mixin>
-
-=item * CPAN Ratings
-
-L<http://cpanratings.perl.org/d/Net-SNMP-Mixin>
-
-=item * RT: CPAN's request tracker
-
-L<http://rt.cpan.org/NoAuth/Bugs.html?Dist=Net-SNMP-Mixin>
-
-=item * Search CPAN
-
-L<http://search.cpan.org/dist/Net-SNMP-Mixin>
-
-=back
+Karl Gaissmaier <karl.gaissmaier at uni-ulm.de>
 
 =head1 COPYRIGHT & LICENSE
 
